require 'json'
require 'securerandom'

module Koa
  module LeaderboardClient
    URL = Conf.env!('LEADERBOARD_URL')
    GAME_ID = Conf.env!('KOA_GAME_ID')

    include Measurement
    measure_methods :get_leaderboards, :get_user_scores, :add_score

    def self.get_leaderboards(limit)
      response = Koa::Request.make(
        type: :get,
        url: URL + "/boards",
        data: {limit: limit, game_id: GAME_ID}
      )
      if response.successful?
        JSON.parse(response.body)
      else
        {}
      end
    end

    def self.get_user_scores(user_id)
      response = Koa::Request.make(
        type: :get,
        url: URL + "/user_scores",
        data: {game_id: GAME_ID, user_id: user_id}
      )
      JSON.parse(response.body) if response.successful?
    end

    def self.add_score(user_id, score)
      response = Koa::Request.make(
        type: :put,
        url: URL + "/score",
        data: {
          user_id: user_id,
          score: score,
          request_id: SecureRandom.uuid,
          game_id: GAME_ID
        },
        tries: 3,
        timeout: 5
      )
      JSON.parse(response.body)["scores"] if response.successful?
    end

  end
end
