module Koa
  module Measurement
    def self.included(base)
      base.extend ClassMethods
    end

    module ClassMethods
      def measurement_name(method)
        (self.name.gsub(/::/, '.').
          gsub(/([A-Z]+)([A-Z][a-z])/,'\1_\2').
          gsub(/([a-z\d])([A-Z])/,'\1_\2').
          tr("-", "_").
          downcase + "." + method.to_s).gsub(/_/,"-")
      end

      def measure_methods(*args)
        @measured_methods ||= []
        @methods_to_measure ||= []
        @methods_to_measure += args
        @methods_to_measure.map!(&:to_sym)
      end

      def method_added(name)
        return if @adding_measurers
        return if @measured_methods.include?(name)
        @adding_measurers = true
        if @methods_to_measure.include? name.to_sym
          unmeasured_name = "unmeasured_#{name}"
          alias_method unmeasured_name, name
          define_method name do |*args, &block|
            Koa::Logger.measure_block(self.class.measurement_name(name)) do
              send unmeasured_name, *args, &block
            end
          end
          @measured_methods << name
        end
        @adding_measurers = nil
      end

      def singleton_method_added(name)
        return if @adding_measurers
        return if @measured_methods.include?(name)
        @adding_measurers = true
        if @methods_to_measure.include? name.to_sym
          unmeasured_name = "unmeasured_#{name}"
          self.singleton_class.send(:alias_method, unmeasured_name, name)
          define_singleton_method name do |*args, &block|
            Koa::Logger.measure_block(measurement_name(name)) do
              send unmeasured_name, *args, &block
            end
          end
          @measured_methods << name
        end
        @adding_measurers = nil
      end
    end
  end
end